/* -----------------------------------------------------------------------------
		UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------

P9. Example of using C language and Atmel microcontrollers to solve logic 
equations or design combinational blocks. 

A Dual_MUX4 combinational circuit.  

Plan: Learn how to set the digital I/O of the microcontroller and how to 
organise the C code in different functions. 

Ref: The circuit is adapted from a previous PIC16F877 version available at:
http://digsys.upc.es/ed//CSD/units/Ch4/U4_05/Unit4_5.html#EX
------------------------------------------------------------------------------*/

#include <avr/io.h>
//#include <stdlib.h>

// Let's see where all the ATmega8535 microcontroller declarations are located:

// 	C:\Program Files (x86)\Atmel\Atmel Studio 6.2\devices\ATmega8535.xml
// 	#include "Atmel Toolchain/AVR8/GCC/Native/3.4.1056/avr8-gnu-toolchain
// 	/avr/include/avr/iom8535.h"


/* ============================================================================
	Definitions 
   ========================================================================== */

//#define	PORTBIT(adr, bit) ( (unsigned)(&adr)*8 + (bit) )

#define Enable_pin_PORT	PINA	// The port where the input "Enable" is located
#define Enable_pin	PINA5 // The pin number where the input "Enable" is located

/* =============================================================================
	Function prototypes 
   ========================================================================== */
void init_system (void);
void read_inputs (void);
void truth_table (void);
void write_outputs (void);

/* =============================================================================
	Global variables  
  =========================================================================== */
// In order to be able to watch variable values using the debugger,
// Let's define them all conveniently as "Char" (1 Byte long)
char Enable;  

char Data_in; 

char Channel_0; 
char Channel_1;
char Channel_2;
char Channel_3;
char Sel;
char Y_out;  


/* =============================================================================
	Main function 
  =========================================================================== */
int main(void){    // Let's make the main() the simplest possible.
    init_system();
    while(1) {		/* loop forever reading inputs and calculating outputs*/
		read_inputs();
        truth_table();
        write_outputs();
    }		
}	

/* =============================================================================
	Function definitions
   ========================================================================== */

//*****************************************************************************
//Scan inputs and process the bits so that we can set our convenient variables
// many of the variables are "char" because we can easily monitor them using
// the watch window for debugging purposes.
//*****************************************************************************
void read_inputs (void){         

// Let's read the Enable input signal
	if((Enable_pin_PORT & (1<<Enable_pin) ) )  Enable = 1; 
		else Enable = 0;

	Data_in = PINC;    // Read all the PORTC bits in a single statement
	
            // Let's process the channels using a mask and shift operators
	Channel_0 = Data_in & 0b00000011;
		
	Channel_1 = (Data_in & 0b00001100)>> 2;
	
	Channel_2 = (Data_in & 0b00110000)>> 4;
	
	Channel_3 = (Data_in & 0b11000000)>> 6;

    // Let's read the Sel variable
	Sel = (PIND & 0b11000000)>> 6;
}


//*****************************************************************************
// The behavioural description of the circuit operation or the
// data processing algorithm
//*****************************************************************************
void truth_table(void){

// This is the combinational circuit truth table described in a behavioural way:

	if (Sel == 0 && Enable == 1) {
		Y_out = Channel_0;
		}
	else if (Sel == 1 && Enable == 1){
		Y_out = Channel_1;
		}
	else if (Sel == 2 && Enable == 1){
		Y_out = Channel_2;
		}
	else if (Sel == 3 && Enable == 1){
		Y_out = Channel_3;
        }
	else 
	    Y_out = 0x00;  

	
}
//*****************************************************************************
// Driving the output pins  
//*****************************************************************************

void write_outputs(void){
// Let's drive the output ports accordingly to the output variables:
    
	PORTD = Y_out; 
// Better write bit by bit using a mask 
}

//*****************************************************************************
// Initialisation operations  
//*****************************************************************************
void init_system (void){
    // Configuration bits
    // These are the ATMEL ATmega8535 configuration bits 
    // They can be generated automatically in the AVR Studio 
    // and normally you don't have to change them.
    // So, copy and paste them as they are to your projects

/*
//#pragma config FOSC = EXTRC     // Oscillator Selection bits (RC oscillator)
#pragma config FOSC = XT     // External XTAL Oscillator

//#pragma config WDTE = ON        // Watchdog Timer Enable bit (WDT enabled)
#pragma config WDTE = OFF       // Watchdog Timer not in use (WDT disabled)

//#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config PWRTE = ON      // Power-up Timer Enable bit (PWRT enabled)

#pragma config BOREN = ON       // Brown-out Reset Enable bit (BOR enabled)
#pragma config LVP = ON         // Low-Voltage (Single-Supply) In-Circuit Serial 
								// Programming Enable bit (RB3/PGM pin has PGM 
								// function; low-voltage programming enabled)

#pragma config CPD = OFF        // Data EEPROM Memory Code Protection bit (Data 
								// EEPROM code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write 
								// protection off; all program memory may be 
								// written to by EECON control)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit 
								// (Code protection off)

*/    


/*
C Code Example from the datasheet I/O section

The following code example shows how to set port B pins 0 and 1 high, 2 and 3 
low, and define the port pins from 4 to 7 as input with pull-ups assigned to 
port pins 6 and 7. The resulting pin values are read back again, but as 
previously discussed, a nop instruction is included to be able to read back the
value recently assigned to some of the pins.

unsigned char i;
...

// Define pull-ups and set outputs high 
// Define directions for port pins 
PORTB = (1<<PB7)|(1<<PB6)|(1<<PB1)|(1<<PB0);
DDRB = (1<<DDB3)|(1<<DDB2)|(1<<DDB1)|(1<<DDB0);

//Insert nop for synchronization
_NOP();
// Read port pins 

i = PINB;

*/

// Initialise starts here:
    // Pay attention to how the ports are configured using mainly PORTx, 
    // DDRx registers
	
	PORTA 	= 0x00;     // Let's not use the Pull-up resistor in PA5
	//PORTA = (0<<PA5);		 

	//DDRA 	= 0b11011111;		/* PORTA5 is an input (the Enable_pin) */
	DDRA = (1<<DDA5);	/* PORTA5 is an input (the Enable_pin) */

	
	PORTD 	= 0x00;		// Let's not use the Pull-up resistors in PD6 and PD7
	//PORTD = (1<<PB7)|(1<<PB6);	

	DDRD 	= 0b00111111;	// PORTD7 and PORTD6 are inputs
                            // PORTD1 and PORTD0 are outputs
	//DDRB = (1<<DDB1)|(1<<DDB0);

	PORTC 	= 0x00;        // Let's not use the Pull-up resistors
	
	DDRC	= 0x00;         // PORTC are all inputs for the channels

	//GIE 	= 0;	// No interrupts in this very simple application 
	//sei();			// Enable Global Interrupt
	
	Y_out = 0x00;	//Output cleared 
	
}
